/*
 * This file is part of MinimalGS, which is a GameScript for OpenTTD
 * Copyright (C) 2011  Leif Linse
 *
 * MinimalGS is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * MinimalGS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MinimalGS; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/* Import SuperLib for GameScript */
import("util.superlib", "SuperLib", 21);
//Result <- SuperLib.Result;
Log <- SuperLib.Log;
//Helper <- SuperLib.Helper;
//ScoreList <- SuperLib.ScoreList;
//Tile <- SuperLib.Tile;
//Direction <- SuperLib.Direction;
//Town <- SuperLib.Town;
//Industry <- SuperLib.Industry;


class MainClass extends GSController 
{
	industry_pair = null;
	cargo_list = null;
	monitor_keys = null;

	constructor()
	{
		industry_pair = null;
		cargo_list = null;
		monitor_keys = null;
	}
}

function MainClass::Start()
{
	this.Setup();

	// Wait for the game to start
	this.Sleep(1);


	local last_monitor = 0;
	while (true) {
		// Time to monitor?
		local now = GSDate.GetCurrentDate();
		if (last_monitor + GSController.GetSetting("frequency") < now)
		{
			last_monitor = now;

			this.Monitor();
		}

		this.Sleep(5); // sleep some time
	}
}

function MainClass::Setup()
{
	// Get the two industries to monitor
	local industry_pair = this.FindIndustryPair();
	local relation = GetCargoRelation(industry_pair[0], industry_pair[1]);
	this.industry_pair = [relation[0], relation[1]];
	this.cargo_list = relation[2];

	// Show which industries that we monitor
	foreach (cargo, _ in cargo_list)
	{
		GSSubsidy.Create(cargo, 
				GSSubsidy.SPT_INDUSTRY, this.industry_pair[0], 
				GSSubsidy.SPT_INDUSTRY, this.industry_pair[1]);
	}
	GSSign.BuildSign(GSIndustry.GetLocation(this.industry_pair[0]), "source");
	GSSign.BuildSign(GSIndustry.GetLocation(this.industry_pair[1]), "destination");

	// Encode Monitor keys and store in a list
	this.monitor_keys = GSList();
	local company_id = 0; // human player
	foreach (cargo, _ in cargo_list)
	{
		// From industry
		local key = GSCargoMonitor.EncodeMonitor(this.industry_pair[0], company_id, cargo);
		this.monitor_keys.AddItem(key, 0);

		// To industry
		key = GSCargoMonitor.EncodeMonitor(this.industry_pair[1], company_id, cargo);
		this.monitor_keys.AddItem(key, 0);
	}

	// Start monitoring
	foreach (key, _ in this.monitor_keys)
	{
		local industry_id = GSCargoMonitor.GetIndustry(key);
		if (industry_id == this.industry_pair[0])
			GSCargoMonitor.GetPickupAmount(key, true);
		else
			GSCargoMonitor.GetDeliveryAmount(key, true);
	}
}

function MainClass::Monitor()
{
	// Monitor
	local total_pickup = 0;
	local total_delivery = 0;
	foreach (key, _ in this.monitor_keys)
	{
		local industry_id = GSCargoMonitor.GetIndustry(key);
		local pickup = GSCargoMonitor.GetPickupAmount(key, true);
		local delivery = GSCargoMonitor.GetDeliveryAmount(key, true);

		Log.Info("key: " + key + " industry: " + GSIndustry.GetName(industry_id) + " pickup: " + pickup + " delivery: " + delivery);

		if (industry_id == this.industry_pair[0])
			total_pickup += pickup;
		else
			total_delivery += delivery;
	}

	Log.Info("Pickup since last monitor: " + total_pickup, Log.LVL_INFO);
	Log.Info("Delivery since last monitor: " + total_delivery, Log.LVL_INFO);
}

// If its possible to transport ind1 => ind2 or the other way around,
// this function returns an array: [from_industry, to_industry, cargo_list]
//
// The cargo_list is an GSLIst with all cargoes that can be transported from
// from_industry to to_industry.
function GetCargoRelation(ind1, ind2)
{
	// Get cargoes produced/accepted by both industries
	local ind1_prod = GSIndustryType.GetProducedCargo(GSIndustry.GetIndustryType(ind1));
	local ind1_accept = GSIndustryType.GetAcceptedCargo(GSIndustry.GetIndustryType(ind1));

	local ind2_prod = GSIndustryType.GetProducedCargo(GSIndustry.GetIndustryType(ind2));
	local ind2_accept = GSIndustryType.GetAcceptedCargo(GSIndustry.GetIndustryType(ind2));

	// Is at least one cargo from industry 1 accepted by industry 2?
	ind1_prod.KeepList(ind2_accept);
	if (!ind1_prod.IsEmpty()) return [ind1, ind2, ind1_prod];

	// Is at least one cargo from industry 2 accepted by industry 1?
	ind1_accept.KeepList(ind2_prod);
	if (!ind1_accept.IsEmpty()) return [ind2, ind1, ind1_accept];
	return null;
}

function IsCompatibleIndustry(ind1, ind2)
{
	return GetCargoRelation(ind1, ind2) != null;
}


function MainClass::FindIndustryPair()
{
	local industry_list = GSIndustryList();

	// Pick a random industry as first industry

	// Edit: Pick a deterministic pair, so that proper save/load is not needed, but still pick the same pair when loading.

//	industry_list.Valuate(GSBase.RandItem);
//	industry_list.Sort(GSList.SORT_BY_VALUE, GSList.SORT_ASCENDING);
	local ind1 = industry_list.Begin();
	industry_list.RemoveItem(ind1);

	// Pick a random compatible industry
	industry_list.Valuate(IsCompatibleIndustry, ind1);
	industry_list.KeepValue(1);
//	industry_list.Valuate(GSBase.RandItem);
	local ind2 = industry_list.Begin();

	if (!GSIndustry.IsValidIndustry(ind1) ||
			!GSIndustry.IsValidIndustry(ind2))
	{
		throw Exception("Failed to find two compatible industries");
	}

	return [ind1, ind2];
}
